/*============================================================================
  File:     Filtered Statistics.sql

  Summary:  Setup and investigate Filtered Indexes

  Date:     November 2009

  SQL Server Version: 10.0.2531.0 (SQL Server 2008 SP1)
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp & Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended as a supplement to the SQL Server 2008 Jumpstart or
  Metro training.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE [AdventureWorksDW2008_ModifiedSalesKey]
go

sp_helpindex2 FactInternetSales
go

SELECT S.CustomerKey, COUNT(*) AS NumTotalSales
FROM FactInternetSales AS S
GROUP BY S.CustomerKey
ORDER BY NumTotalSales DESC
go

DBCC SHOW_STATISTICS(FactInternetSales, IX_FactInternetSales_CustomerKey)
go

select 61847552 * 5.410084E-05

--RANGE_HI_KEY	RANGE_ROWS	EQ_ROWS		DISTINCT_RANGE_ROWS	AVG_RANGE_ROWS
--11498			--------	30720		--					--------
--11519			280576		36864		20					14028.8

-- So, what does it tell us for any value supplied 
-- BETWEEN 11498 and 11519...
-- Only the expected number of rows - which is 14,029

-- Wait... what if they're NOT evenly distributed??

SELECT S.CustomerKey, COUNT(*) AS NumTotalSales
FROM FactInternetSales AS S
WHERE S.CustomerKey > 11498 and S.CustomerKey < 11519
GROUP BY S.CustomerKey
ORDER BY NumTotalSales DESC
go

-- In this range:
-- 11502 has the highest number of rows: 44032
-- 11512 has the lowest number of rows: 1024

SET STATISTICS IO ON
SET STATISTICS TIME ON
go

SELECT C.CustomerKey, C.LastName, S.SalesAmount
FROM dbo.FactInternetSales AS S
	INNER JOIN dbo.DimCustomer AS C
		ON S.CustomerKey = C.CustomerKey
WHERE C.CustomerKey = 11502
go

SELECT C.CustomerKey, C.LastName, S.SalesAmount
FROM dbo.FactInternetSales AS S
	INNER JOIN dbo.DimCustomer AS C
		ON S.CustomerKey = C.CustomerKey
WHERE C.CustomerKey = 11512
go

------------------------------------------------------------------------------
-- Create a filtered statistics to help better define a range
------------------------------------------------------------------------------
DBCC SHOW_STATISTICS (FactInternetSales, [SalesByCustomer_R11450-11550])
go

CREATE STATISTICS [SalesByCustomer_R11350-11550] 
	ON FactInternetSales (CustomerKey)
WHERE CustomerKey >= 11350 
	AND CustomerKey <= 11550
go

sp_recompile FactInternetSales
go

-- Do the query plans change?

SELECT C.CustomerKey, C.LastName, S.SalesAmount
FROM dbo.FactInternetSales AS S
	INNER JOIN dbo.DimCustomer AS C
		ON S.CustomerKey = C.CustomerKey
WHERE C.CustomerKey = 11502
go

SELECT C.CustomerKey, C.LastName, S.SalesAmount
FROM dbo.FactInternetSales AS S
	INNER JOIN dbo.DimCustomer AS C
		ON S.CustomerKey = C.CustomerKey
WHERE C.CustomerKey = 11512
go

-- Clean up:
DROP STATISTICS FactInternetSales.[SalesByCustomer_R11450-11550]
go